<?php
/*  Copyright 2013  Plugin Gnome  (email : support@ggnome.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, version 2, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

defined('_JEXEC') or die;
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.archive');
require_once (dirname( __FILE__ ) . '/include/ggsw-include.php');

$document = JFactory::getDocument();
$document->addScript(JURI::base() . 'plugins/content/ggpkg_import/include/swfobject.js');
$document->addScript(JURI::base() . 'plugins/content/ggpkg_import/include/pano2vrgyro.js');

class plgContentGGPKG_Import extends JPlugin {

	public function addLogger()
	{
		$option = array(
				'format'	=> '{DATE}\t{TIME}\t{LEVEL}: {MESSAGE}',
				'text_file' => 'ggsw_log.txt',
				'logger'	=> 'formattedtext');
		JLog::addLogger($option);
	}

	private function writeLogfile($msg, $severity)
	{
		$logfile = $this->params->get('logfile');
		if ($logfile == 'true' || $logfile == '1')
		{
			$this->addLogger();
			JLog::add($msg, $severity);
		}
	}
	
	private function fileInPackage($folder, $file)
	{
		return JFile::exists($folder . "/" . $file);
	}
	
	private function urlFromLocal($url)
	{
		$basepath = JPATH_BASE;
		if (JURI::root(true))
			$basepath = substr($basepath, 0, strpos($basepath, JURI::root(true)));
		
		return JString::substr($url, strlen($basepath));
	}
		
	public function onContentAfterSave($context, &$article, $isNew)
	{
		if ($context == 'com_media.file')
		{
			$file = $article->filepath;
			if ($file && JFile::getExt($file)=="ggpkg" || JFile::getExt($file)=="GGPKG")
			{
				$this->writeLogfile("GGPKG '" . JFile::getName($file) . "'uploaded!", JLog::INFO);
				$ggpkgPath = JPATH_ROOT . "/media/ggpkg";
				if (!JFolder::exists($ggpkgPath))
					JFolder::create($ggpkgPath, 0777);
				$extract_path = $ggpkgPath . "/" . JFile::getName($file);
				if (JArchive::getAdapter('zip')->extract($file, $extract_path))
				{
					$this->writeLogfile("'" . JFile::getName($file) . "' successfully extracted!", JLog::INFO);
					ggsw_chmod($extract_path, 0777);
					
					$package_title = JFile::stripExt(JFile::getName($file));
					$package_title = str_replace(' ', '_', $package_title);
					if (is_numeric(substr($package_title, 0, 1)))
						$package_title = '_' . $package_title;
					if ($this->fileInPackage($extract_path, "pano2vr_player.js"))
					{
						$uniquePlayerVar = "var " . $package_title . "_pano2vrPlayer = pano2vrPlayer;";
						$playerFile = $extract_path . "/pano2vr_player.js";
						file_put_contents($playerFile, $uniquePlayerVar, FILE_APPEND);
						if ($this->fileInPackage($extract_path, "skin.js"))
						{
							$uniqueSkinVar = "var " . $package_title . "_pano2vrSkin = pano2vrSkin;";
							$skinFile = $extract_path . "/skin.js";
							file_put_contents($skinFile, $uniqueSkinVar, FILE_APPEND);
						}
					}
					if ($this->fileInPackage($extract_path, "object2vr_player.js"))
					{
						$uniquePlayerVar = "var " . $package_title . "_object2vrPlayer = object2vrPlayer;";
						$playerFile = $extract_path . "/object2vr_player.js";
						file_put_contents($playerFile, $uniquePlayerVar, FILE_APPEND);
						if ($this->fileInPackage($extract_path, "skin.js"))
						{
							$uniqueSkinVar = "var " . $package_title . "_object2vrSkin = object2vrSkin;";
							$skinFile = $extract_path . "/skin.js";
							file_put_contents($skinFile, $uniqueSkinVar, FILE_APPEND);
						}
					}
				}
				else
					$this->writeLogfile("Error extracting '" . JFile::getName($file) . "'!", JLog::CRITICAL);
			}
		}
	}
	
	public function onContentAfterDelete($context, $article)
	{
		if ($context == 'com_media.file')
		{
			$file = $article->filepath;
			$ggpkgPath = JPATH_ROOT . "/media/ggpkg";
			if ($file && JFile::getExt($file)=="ggpkg" || JFile::getExt($file)=="GGPKG")
			{
				$this->writeLogfile("GGPKG '" . JFile::getName($file) . "'deleted!", JLog::INFO);
				
				if (JFolder::delete($ggpkgPath . "/" . JFile::getName($file)))
					$this->writeLogfile("Temprary folder '" . $ggpkgPath . "/" . JFile::getName($file) . "' successfully deleted!", JLog::INFO);
				else
					$this->writeLogfile("Error deleting folder '" . $ggpkgPath . "/" . JFile::getName($file) . "'!", JLog::CRITICAL);
			}
		}
	}
	
	public function onContentPrepare($context, &$article, &$params, $limitstart) 
	{
		if ($context == 'com_content.article' || $context == 'com_content.featured')
		{
			$current_index=0;
			$start_shortcode=0;
			while ($current_index = JString::strpos(JString::strtoupper($article->text), "{GGPKG ", $current_index))
			{
				if ($closing_bracket = JString::strpos($article->text, "}", $current_index))
				{
					$shortcode = JString::substr($article->text, $current_index, $closing_bracket - $current_index + 1);
					$this->writeLogfile("Shortcode '" . $shortcode . "' found!", JLog::DEBUG);
					$attributes = JUtility::parseAttributes($shortcode);
					if (isset($attributes['file']))
					{
						$file = $attributes['file'];
						if (strcasecmp(substr($file, strlen($file)-6),'.ggpkg')!=0)
							$file.='.ggpkg';
						$confObject = JFactory::getApplication();
						$tmpPath = $confObject->getCfg('tmp_path');
						$ggpkgPath = JPATH_ROOT . "/media/ggpkg";
						$package_path = $tmpPath . "/" . $file; // legacy version
						$folder_found = true;
						if (!JFolder::exists($package_path))
						{
							$package_path = $ggpkgPath . "/" . $file;
							if (!JFolder::exists($package_path))
								$folder_found = false;
						}
						if (!$folder_found)
						{
							$this->writeLogfile("Temporary folder for '" . $file . "' not found!", JLog::INFO);
							$replaceHTML =  "Temporary folder not found!";
						}
						else
						{
							$attributes['package_path'] = $package_path;
							$this->prepareAttributes($attributes);
							$replaceHTML = ggsw_ggpkg_html($attributes);
						}
					}
					else if (isset($attributes['url']))
					{
						$attributes['package_path'] = $attributes['url'];
						$this->prepareAttributes($attributes);
						$replaceHTML = ggsw_ggpkg_html($attributes);
					}
					else
					{
						$lang = JFactory::getLanguage();
						$lang->load('plg_content_ggpkg_import');
						$replaceHTML = JText::_("GGPKG_PLUGIN_ERROR_FILE_ATTRIBUTE");
					}
					
					$article->text = JString::substr($article->text, 0, $current_index) . $replaceHTML . JString::substr($article->text, $closing_bracket + 1); 
					$current_index = $current_index + strlen($replaceHTML);
				}
				else 
					return;
			}
		}
	}
	
	private function addGoogleMapsScript()
	{
		$document = JFactory::getDocument();
		$document->addScript('https://maps.googleapis.com/maps/api/js?v=3.exp');
	}

	private function getHttpResponseCode($url)
	{
		$headers = get_headers($url);
		return substr($headers[0], 9, 3);
	}
	
	private function remoteFileAvailable($url)
	{
		if(intval($this->getHttpResponseCode($url)) < 400)
			return true;
		else
			return false;
	}
	
	private function parseGginfoJson($json_content, $package_path)
	{
		$json = json_decode($json_content);
		$json_externals = $json->{'external'};
		if (isset($json_externals))
		{
			$document = JFactory::getDocument();
			$js_files = $json_externals->{'js'};
			if (isset($js_files))
			{
				foreach ($js_files as $js_file)
				{
                    if (substr($js_file, 0, 4) === "http")
                    {
                        $document->addScript($js_file);
                    }
                    else
                    {
                        $document->addScript($this->urlFromLocal($package_path) . "/" .$js_file);
                    }
				}
			}
			$css_files = $json_externals->{'css'};
			if (isset($css_files))
			{
				foreach ($css_files as $css_file)
				{
                    if (substr($css_file, 0, 4) === "http")
                    {
                        $document->addStyleSheet($css_file);
                    }
                    else
                    {
                        $document->addStyleSheet($this->urlFromLocal($package_path) . "/" .$css_file);
                    }
				}
			}
		}
	}
	
	private function parseGginfoFile($config_file, $package_path)
	{
		if (($json_content = file_get_contents($config_file)) != false)
		{
			$json_content = utf8_encode($json_content);
			$this->parseGginfoJson($json_content, $package_path);
		}
	}
	
	private function prepareAttributes(&$attributes)
	{
		$confObject = JFactory::getApplication();
		$tmpPath = $confObject->getCfg('tmp_path');
		$ggpkgPath = JPATH_ROOT . "/media/ggpkg";
		
		$width = $this->params->get('width');
		$height = $this->params->get('height');
		if (!isset($attributes['width']))
			$attributes['width'] = trim($width);
		if (!isset($attributes['height']))
			$attributes['height'] = trim($height);

		$attributes['gyro_file'] = $this->urlFromLocal(dirname(__FILE__) . "include/pano2vrgyro.js");
		$gyro = $this->params->get('gyro');
		if (!isset($attributes['gyro']))
			$attributes['gyro'] = $gyro;
		
		$gyro_north = $this->params->get('gyro_north');
		if (!isset($attributes['gyro_north']))
			$attributes['gyro_north'] = $gyro_north;
			
		$start_preview = $this->params->get('start_preview');
		if (!isset($attributes['start_preview']))
			$attributes['start_preview'] = $start_preview;
					
		if (isset($attributes['url']))
		{
			$url = $attributes['url'];
			if (substr($url, -1) != '/')
				$url .= '/';
			
			$attributes['package_url'] = $url;
			$md5_hash = md5($attributes['url']);
			if (is_numeric(substr($md5_hash, 0, 1)))
				$md5_hash = '_' . $md5_hash;
			$package_title = $md5_hash;
			$attributes['package_title'] = $package_title;
			$attributes['id'] = $package_title;
				
			$remote_json_file = $attributes['package_url'] . "gginfo.json";
			if ($this->remoteFileAvailable($remote_json_file))
			{
				$attributes['async_loading'] = 'true';
				if (function_exists('curl_version')) // curl available
				{
					$curl = curl_init();
					curl_setopt($curl, CURLOPT_URL, $remote_json_file);
					curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
					curl_setopt($curl, CURLOPT_HEADER, false);
					$json_content = curl_exec($curl);
					curl_close($curl);
				}
				else
				{
					$json_content = file_get_contents($remote_json_file);
				}
		
				$this->parseGginfoJson($json_content, $attributes['package_path']);
				$json = json_decode($json_content);
				$json_type = $json->{'type'};
				if (isset($json_type))
				{
					if ($json_type == 'panorama')
					{
						$attributes['is_pano'] = 'true';
						$attributes['xml_file'] = $url . "pano.xml";
						$attributes['xml_file_local'] = $url . "pano.xml";
					}
					else
					{
						$attributes['is_pano'] = 'false';
						$attributes['xml_file'] = $url . "object.xml";
						$attributes['xml_file_local'] = $url . "object.xml";
					}
				}
				$json_player = $json->{'player'};
				if (isset($json_player))
				{
					$json_js_player = $json_player->{'js'};
					if (isset($json_js_player))
					{
						$attributes['player_file'] = $url . $json_js_player;
					}
				}
				$json_skin = $json->{'skin'};
				if (isset($json_skin))
				{
					$json_skin_file = $json_skin->{'js'};
					if (isset($json_skin_file))
					{
						$attributes['has_skin'] = 'true';
						$attributes['skin_file'] = $url . $json_skin_file;
					}
				}
				$json_preview = $json->{'preview'};
				if (isset($json_preview))
				{
					$json_preview_file = $json_preview->{'img'};
					if (isset($json_preview_file))
					{
						$attributes['preview_file'] = $url . $json_preview_file;
						$attributes['preview_play_file'] = $this->urlFromLocal(dirname(__FILE__) . "/include/play.png");
					}
					else
					{
						$attributes['start_preview'] = 'false';
					}
				}
				else
				{
					$attributes['start_preview'] = 'false';
				}
			}
			else
			{
				$this->writeLogfile("No gginfo.json found at the remote url!", JLog::CRITICAL);
			}
		}
		else
		{
			$attributes['id'] = md5($attributes['file']);
			
			$attributes['package_url'] = $this->urlFromLocal($attributes['package_path']);
			$package_title = JFile::stripExt(JFile::getName($attributes['package_path']));
			if (is_numeric(substr($package_title, 0, 1)))
				$package_title = '_' . $package_title;
			$attributes['package_title'] = $package_title;
		
			// read config file
			if ($this->fileInPackage($attributes['package_path'], "gginfo.json"))
			{
				$config_file = $attributes['package_path'] . "/gginfo.json";
				$this->parseGginfoFile($config_file, $attributes['package_path']);
				$attributes['async_loading'] = 'true';
			}
		
			if ($attributes['start_preview'] == 'true' || $attributes['start_preview'] == '1')
			{
				$attributes['preview_file'] = $this->urlFromLocal($attributes['package_path'] . "/preview.jpg");
				$attributes['preview_play_file'] = $this->urlFromLocal(dirname(__FILE__) . "/include/play.png");
			}
		
			if ($this->fileInPackage($attributes['package_path'], "skin.js"))
			{
				$attributes['has_skin'] = 'true';
				$attributes['skin_file'] = $this->urlFromLocal($attributes['package_path'] . "/skin.js");
			}
			else
				$attributes['has_skin'] = 'false';
			
			$attributes['is_pano'] = 'false';
			if ($this->fileInPackage($attributes['package_path'], "pano.xml"))
			{
				$attributes['player_file'] = $this->urlFromLocal($attributes['package_path'] . "/pano2vr_player.js");
				$attributes['xml_file'] = $this->urlFromLocal($attributes['package_path'] . "/pano.xml");
				$attributes['xml_file_local'] = $attributes['package_path'] . "/pano.xml";
				$attributes['is_pano'] = 'true';
			}
			else if ($this->fileInPackage($attributes['package_path'], "object.xml"))
			{
				$attributes['player_file'] = $this->urlFromLocal($attributes['package_path'] . "/object2vr_player.js");
				$attributes['xml_file'] = $this->urlFromLocal($attributes['package_path'] . "/object.xml");
				$attributes['xml_file_local'] = $attributes['package_path'] . "/object.xml";
				$attributes['is_pano'] = 'false';
			}
		}

		if (!is_null($this->params->get('display_map')) && $this->params->get('display_map')=='1' && $attributes['is_pano'] == 'true')
		{
			$display_map = true;
		}
		else 
		{
			$display_map = false;
		}
		if (!isset($attributes['display_map']))
			$attributes['display_map'] = $display_map;
		if ($attributes['is_pano'] != 'true')
			$attributes['display_map'] = 'false';
		
		if ((isset($attributes['map_only']) && ($attributes['map_only'] == 'true' || $attributes['map_only'] == '1'))
			|| $attributes['display_map'] == 'true')
			$this->addGoogleMapsScript();
			
		if (!is_null($this->params->get('display_userdata')) && $this->params->get('display_userdata')=='1')
			$display_userdata = true;
		else 
			$display_userdata = false;
		if (!isset($attributes['display_userdata']))
			$attributes['display_userdata'] = $display_userdata;
		
		$attributes['userdata_html'] = $this->params->get('userdata');
		
		if (!is_null($this->params->get('masterpanorama')) && $this->params->get('masterpanorama') != "")
		{
			$file = $this->params->get('masterpanorama');
			if (strcasecmp(substr($file, strlen($file)-6),'.ggpkg')!=0)
				$file.='.ggpkg';
			$master_panorama_dir = $tmpPath . "/" . $file;
			if (!JFolder::exists($master_panorama_dir))
				$master_panorama_dir = $ggpkgPath . "/" . $file;
			$attributes['masterpanorama_dir'] = $this->urlFromLocal($master_panorama_dir);
			if (!is_null($this->params->get('player_from_masterpanorama')) && $this->params->get('player_from_masterpanorama')=='1')
			{
				if (JFile::exists($master_panorama_dir . "/pano2vr_player.js"))
				{
					$attributes['player_file_from_masterpanorama'] = $this->urlFromLocal($master_panorama_dir . "/pano2vr_player.js");
					$attributes['player_from_masterpanorama'] = 'true';
				}
			}
			if (!is_null($this->params->get('skin_from_masterpanorama')) && $this->params->get('skin_from_masterpanorama') == '1')
			{
				if (JFile::exists($master_panorama_dir . "/skin.js"))
				{
					$attributes['skin_file_from_masterpanorama'] = $this->urlFromLocal($master_panorama_dir . "/skin.js");
					$attributes['skin_from_masterpanorama'] = 'true';
					$attributes['has_skin'] = 'true';
				}
			}
		}
			
		if (!is_null($this->params->get('masterobject')) && $this->params->get('masterobject') != "")
		{
			$file = $this->params->get('masterobject');
			if (strcasecmp(substr($file, strlen($file)-6),'.ggpkg')!=0)
				$file.='.ggpkg';
			$master_object_dir = $tmpPath . "/" . $file;
			if (!JFolder::exists($master_object_dir))
				$master_object_dir = $ggpkgPath . "/" . $file;
			$attributes['masterobject_dir'] = $this->urlFromLocal($master_object_dir);
			if (!is_null($this->params->get('player_from_masterobject')) && $this->params->get('player_from_masterpanorama')=='1')
			{
				if (JFile::exists($master_object_dir . "/object2vr_player.js"))
				{
					$attributes['player_file_from_masterobject'] = $this->urlFromLocal($master_object_dir . "/object2vr_player.js");
					$attributes['player_from_masterobject'] = 'true';
				}
			}
			if (!is_null($this->params->get('skin_from_masterobject')) && $this->params->get('skin_from_masterobject') == '1')
			{
				if (JFile::exists($master_object_dir . "/skin.js"))
				{
					$attributes['skin_file_from_masterobject'] = $this->urlFromLocal($master_object_dir . "/skin.js");
					$attributes['skin_from_masterobject'] = 'true';
					$attributes['has_skin'] = 'true';
				}
			}
		}
		
		$lang = JFactory::getLanguage();
		$lang->load('plg_content_ggpkg_import');
		$attributes['error_missing_xml'] = JText::_("GGPKG_PLUGIN_ERROR_MISSING_XML");
		$attributes['error_html5_required'] = JText::_("GGPKG_PLUGIN_ERROR_HTML5_REQUIRED");
		$attributes['error_javascript_required'] = JText::_("GGPKG_PLUGIN_ERROR_JAVASCRIPT_REQUIRED");
	}
}
